/* Declarations of functions and data types used for SHA1 sum
 library functions.
 Copyright (C) 2000, 2001, 2003, 2005 Free Software Foundation, Inc.

 This program is free software; you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by the
 Free Software Foundation; either version 2, or (at your option) any
 later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software Foundation,
 Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.  */

/*
 This file has been modified from the original found in http://www.gnu.org/software/coreutils/ coreutils-5.97 for use within AtomicParsley.
 Modifications are : md5_uint32 typedef
 file renaming
 compiler attribute directives eliminated to msvc
 */

#include <stdio.h>

#include "AP_commons.h"

typedef uint32_t md5_uint32;

/* Structure to save state of computation between the single steps.  */
struct sha1_ctx {
    md5_uint32 A;
    md5_uint32 B;
    md5_uint32 C;
    md5_uint32 D;
    md5_uint32 E;

    md5_uint32 total[2];
    md5_uint32 buflen;
    char buffer[128]; //char buffer[128] __attribute__ ((__aligned__ (__alignof__ (md5_uint32))));
};

/* Initialize structure containing state of computation. */
extern void sha1_init_ctx(struct sha1_ctx *ctx);

/* Starting with the result of former calls of this function (or the
 initialization function update the context for the next LEN bytes
 starting at BUFFER.
 It is necessary that LEN is a multiple of 64!!! */
extern void sha1_process_block(const void *buffer, size_t len, struct sha1_ctx *ctx);

/* Starting with the result of former calls of this function (or the
 initialization function update the context for the next LEN bytes
 starting at BUFFER.
 It is NOT required that LEN is a multiple of 64.  */
extern void sha1_process_bytes(const void *buffer, size_t len, struct sha1_ctx *ctx);

/* Process the remaining bytes in the buffer and put result from CTX
 in first 20 bytes following RESBUF.  The result is always in little
 endian byte order, so that a byte-wise output yields to the wanted
 ASCII representation of the message digest.

 IMPORTANT: On some systems it is required that RESBUF be correctly
 aligned for a 32 bits value.  */
extern void *sha1_finish_ctx(struct sha1_ctx *ctx, void *resbuf);

/* Put result from CTX in first 20 bytes following RESBUF.  The result is
 always in little endian byte order, so that a byte-wise output yields
 to the wanted ASCII representation of the message digest.

 IMPORTANT: On some systems it is required that RESBUF is correctly
 aligned for a 32 bits value.  */
extern void *sha1_read_ctx(const struct sha1_ctx *ctx, void *resbuf);

/* Compute SHA1 message digest for bytes read from STREAM.  The
 resulting message digest number will be written into the 20 bytes
 beginning at RESBLOCK.  */
extern int sha1_stream(FILE *stream, void *resblock);

/* Compute SHA1 message digest for LEN bytes beginning at BUFFER.  The
 result is always in little endian byte order, so that a byte-wise
 output yields to the wanted ASCII representation of the message
 digest.  */
extern void *sha1_buffer(const char *buffer, size_t len, void *resblock);
